/*
 *  Creature.cpp
 *  emptyExample
 *
 *  Created by Andrew Wallace on 9/13/11.
 *  Copyright 2011 Cool Town Inc. All rights reserved.
 *
 */



#include "Creature.h"

//--------------------------------------------------------------
void Creature::setup(){
	
	
	
	//fill up the position array with a starting point
	ofPoint startPos;
	startPos.x=0;
	startPos.y=0;
	for (int i=0; i<100; i++)	pos.push_back(startPos);
	
	//set up the segments and the head
	for (int i=0; i<SEG_NUM; i++)	segments[i].setup();
	head.setup();
}

//--------------------------------------------------------------
void Creature::update(float mX, float mY){
	
	//move the head toward the goal
	head.update(mX,mY);
	
	//move the segments along, following the head
	for (int i=0; i<SEG_NUM; i++){
		ofPoint prevPos;	//position of the segment (or head) in front of this one
		if (i==0)	prevPos=head.pos;
		else		prevPos=segments[i-1].pos;
		
		//need to calulate the angle
		float angle=atan2(prevPos.y-segments[i].pos.y, prevPos.x-segments[i].pos.x);
		segments[i].update( pos[pos.size()-(i+1)*5], angle);
	}
	
	//set the body angle of the head
	head.bodyAngle=segments[0].angle;
	
	
	//add the new head position to the vector
	pos.push_back(head.pos);
	//and get rid of the oldest one
	pos.erase(pos.begin());
	
}

//--------------------------------------------------------------
void Creature::draw(){
	ofFill();
	//draw connections between the segments
	ofSetLineWidth(3);
	ofSetColor(4,133,157,200);
	for (int i=0; i<SEG_NUM; i++){
		ofPoint prevPos;	//position of the segment (or head) in front of this one
		if (i==0)	prevPos=head.pos;
		else		prevPos=segments[i-1].pos;
		
		ofLine(segments[i].pos.x, segments[i].pos.y, prevPos.x, prevPos.y);
		
	}
	
	//draw the sgments and head
	for (int i=0; i<SEG_NUM; i++)	segments[i].draw();
	head.draw();
	
	
	
}