#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	ofEnableAlphaBlending();
	showImg=false;
	width=1152;
	height=864;


	#ifdef _USE_LIVE_VIDEO
        vidGrabber.setVerbose(true);
        vidGrabber.initGrabber(width,height);
	#else
        vidPlayer.loadMovie("fingers.mov");
        vidPlayer.play();
	#endif

    colorImg.allocate(width,height);
	grayImage.allocate(width,height);
	grayBg.allocate(width,height);
	grayDiff.allocate(width,height);

	bLearnBakground = true;
	threshold = 50;
	
	counter=0;
	
	blobX=ofGetWidth()/2;
	blobY=ofGetWidth()/2;
	targetX=blobX;
	targetY=blobY;
	
	creature.setup();
}

//--------------------------------------------------------------
void testApp::update(){
	ofBackground(100,100,100);
	
	counter++;	//incriment the counter

    bool bNewFrame = false;

	#ifdef _USE_LIVE_VIDEO
       vidGrabber.grabFrame();
	   bNewFrame = vidGrabber.isFrameNew();
    #else
        vidPlayer.idleMovie();
        bNewFrame = vidPlayer.isFrameNew();
	#endif

	if (bNewFrame){

		#ifdef _USE_LIVE_VIDEO
            colorImg.setFromPixels(vidGrabber.getPixels(), width,height);
	    #else
            colorImg.setFromPixels(vidPlayer.getPixels(), width,height);
        #endif

		grayImage = colorImg;
		if (bLearnBakground == true){
			grayBg = grayImage;		// the = sign copys the pixels from grayImage into grayBg (operator overloading)
			bLearnBakground = false;
		}

		// take the abs value of the difference between background and incoming and then threshold:
		grayDiff.absDiff(grayBg, grayImage);
		grayDiff.threshold(threshold);

		// find contours which are between the size of 20 pixels and 1/3 the w*h pixels.
		// also, find holes is set to true so we will get interior contours as well....
		contourFinder.findContours(grayDiff, 20, (340*239)/3, 10, false);	// find holes
		
		//find the biggest blob
		if(contourFinder.blobs.size() > 0) {
			int len = contourFinder.blobs.size() - 1;
			int i=0;
			largestBlob=0;
			while(i<len){
				if(largestBlob == -1) {
					largestBlob = i;
				} else if(contourFinder.blobs.at(i).area > contourFinder.blobs.at(largestBlob).area) {
					largestBlob = i;
				}
				i++;
			}
			//save the middle of the largest blog as the target
			blobX=contourFinder.blobs[largestBlob].boundingRect.x+contourFinder.blobs[largestBlob].boundingRect.width/2;
			blobY=contourFinder.blobs[largestBlob].boundingRect.y+contourFinder.blobs[largestBlob].boundingRect.height/2;
		}
		
	}
	
	//xeno toward the new target
	float speed=0.1;
	targetX=(1-speed)*targetX + speed*blobX;
	targetY=(1-speed)*targetY + speed*blobY;
	creature.update(targetX,targetY);
	
	//every so often, snap a new background image to keep up if the environment changes
	if (counter==3000){
		cout<<"snap"<<endl;
		bLearnBakground=true;
		counter=0;
	}

}

//--------------------------------------------------------------
void testApp::draw(){

	// draw the incoming, the grayscale, the bg and the thresholded difference
	ofSetColor(0xffffff);
	colorImg.draw(0,0);

	
	////show the target and blob center
//	ofSetColor(255, 0, 0);
//	ofNoFill();
//	ofCircle(blobX, blobY, 15);
//	ofCircle(targetX, targetY, 10);

	// finally, a report:
	ofSetColor(0xffffff);
	char reportStr[1024];
	sprintf(reportStr, "press Space to capture bg\nthreshold %i (press: +/-)\nnum blobs found %i, fps: %f", threshold, contourFinder.nBlobs, ofGetFrameRate());
	ofDrawBitmapString(reportStr, 20, ofGetHeight()-50);
	

	creature.draw();
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){

	switch (key){
		case ' ':
			bLearnBakground = true;
			counter=0;
			break;
		case '+':
			threshold ++;
			if (threshold > 255) threshold = 255;
			break;
		case '-':
			threshold --;
			if (threshold < 0) threshold = 0;
			break;
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	showImg=true;

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

